expected_result = (
    '<paragraph>D.\xa0Lindley. <emphasis>Making Decisions</emphasis>. '
    'Wiley, 2nd edition, 1985.</paragraph>'
    )


def test_install_example():
    result = []

    # example begin
    import io
    import pybtex.database.input.bibtex
    import pybtex.plugin

    style = pybtex.plugin.find_plugin('pybtex.style.formatting', 'plain')()
    backend = pybtex.plugin.find_plugin('pybtex.backends', 'docutils')()
    parser = pybtex.database.input.bibtex.Parser()
    data = parser.parse_stream(io.StringIO(u"""
    @Book{1985:lindley,
      author =    {D. Lindley},
      title =     {Making Decisions},
      publisher = {Wiley},
      year =      {1985},
      edition =   {2nd},
    }
    """))
    for entry in style.format_entries(data.entries.values()):
        print(backend.paragraph(entry))
    # example end
        result.append(backend.paragraph(entry))

    assert len(result) == 1
    assert str(result[0]) == expected_result



# note: tests require pybtex-docutils to be installed
# so its entry points can be found

import pybtex_docutils


def test_pkg_resources_entry_point():
    from pkg_resources import iter_entry_points
    for ep in iter_entry_points("pybtex.backends", "docutils"):
        assert ep.load() is pybtex_docutils.Backend


def test_pybtex_find_plugin():
    from pybtex.plugin import find_plugin
    assert (
        find_plugin("pybtex.backends", "docutils") is pybtex_docutils.Backend)



"""
API
~~~

The backend renders :class:`pybtex.richtext.Text` instances
into a list of :class:`docutils.nodes.Node` instances.
For typical use cases, all you need to care about are the methods
:meth:`Backend.paragraph`,
:meth:`Backend.citation`, and
:meth:`Backend.citation_reference`
which are to be called on *formatted* entries,
as in the :ref:`minimal example <minimal-example>`.

Unless you are subclassing :class:`Backend` to create a new backend,
you should normally not import the :mod:`pybtex_docutils` module directly.
Instead, use pybtex's plugin system to get the :class:`Backend` class,
as in the :ref:`minimal example <minimal-example>`.

.. autoclass:: Backend
   :show-inheritance:
   :members: RenderType, paragraph, citation, citation_reference,
             footnote, footnote_reference
"""

import docutils.nodes
import itertools

from pybtex.backends import BaseBackend
from typing import TYPE_CHECKING, List, Type

if TYPE_CHECKING:
    from pybtex.style import FormattedEntry


class Backend(BaseBackend):
    name = 'docutils'

    symbols = {
        'ndash': [docutils.nodes.Text('\u2013', '\u2013')],
        'newblock': [docutils.nodes.Text(' ', ' ')],
        'nbsp': [docutils.nodes.Text('\u00a0', '\u00a0')],
    }
    tags = {
        'emph': docutils.nodes.emphasis,  # note: deprecated
        'em': docutils.nodes.emphasis,
        'strong': docutils.nodes.strong,
        'i': docutils.nodes.emphasis,
        'b': docutils.nodes.strong,
        'tt': docutils.nodes.literal,
        'sup': docutils.nodes.superscript,
        'sub': docutils.nodes.subscript,
    }

    RenderType: Type[List[docutils.nodes.Node]] = list

    # for compatibility only
    def format_text(self, text: str) -> List[docutils.nodes.Node]:
        return self.format_str(text)

    def format_str(self, str_: str) -> List[docutils.nodes.Node]:
        return [docutils.nodes.Text(str_, str_)]

    def format_tag(self, tag_name: str, text: List[docutils.nodes.Node]
                   ) -> List[docutils.nodes.Node]:
        if tag_name in self.tags:
            tag = self.tags[tag_name]
            return [tag('', '', *text)]
        else:
            return text

    def format_href(self, url: str, text: List[docutils.nodes.Node]
                    ) -> List[docutils.nodes.Node]:
        node = docutils.nodes.reference('', '', *text, refuri=url)
        return [node]

    def write_entry(self, key: str, label: str, text: str) -> None:
        raise NotImplementedError("use Backend.citation() instead")

    def render_sequence(self, rendered_list: List[List[docutils.nodes.Node]]
                        ) -> List[docutils.nodes.Node]:
        return list(itertools.chain(*rendered_list))

    def paragraph(self, entry: "FormattedEntry") -> docutils.nodes.paragraph:
        """Return a docutils.nodes.paragraph
        containing the rendered text for *entry* (without label).

        .. versionadded:: 0.2.0
        """
        return docutils.nodes.paragraph('', '', *entry.text.render(self))

    def citation(self, entry: "FormattedEntry",
                 document: docutils.nodes.document, use_key_as_label=True
                 ) -> docutils.nodes.citation:
        """Return citation node, with key as name, label as first
        child, and paragraph with entry text as second child. The citation is
        expected to be inserted into *document* prior to any docutils
        transforms.
        """
        # see docutils.parsers.rst.states.Body.citation()
        if use_key_as_label:
            label = entry.key
        else:
            label = entry.label
        name = docutils.nodes.fully_normalize_name(entry.key)
        citation = docutils.nodes.citation()
        citation['names'].append(name)
        citation += docutils.nodes.label('', label)
        citation += self.paragraph(entry)
        document.note_citation(citation)
        document.note_explicit_target(citation, citation)
        return citation

    def citation_reference(
            self, entry: "FormattedEntry",
            document: docutils.nodes.document, use_key_as_label=True
            ) -> docutils.nodes.citation_reference:
        """Return citation_reference node to the given citation. The
        citation_reference is expected to be inserted into *document*
        prior to any docutils transforms.
        """
        # see docutils.parsers.rst.states.Body.footnote_reference()
        if use_key_as_label:
            label = entry.key
        else:
            label = entry.label
        refname = docutils.nodes.fully_normalize_name(entry.key)
        refnode = docutils.nodes.citation_reference(
            '[%s]_' % label, refname=refname)
        refnode += docutils.nodes.Text(label)
        document.note_citation_ref(refnode)
        return refnode

    def footnote(self, entry: "FormattedEntry",
                 document: docutils.nodes.document) -> docutils.nodes.footnote:
        """Return footnote node, with key as name, and paragraph with
        entry text as child. The footnote is expected to be
        inserted into *document* prior to any docutils transforms.

        .. versionadded:: 0.2.2
        """
        # see docutils.parsers.rst.states.Body.footnote()
        name = docutils.nodes.fully_normalize_name(entry.key)
        footnote = docutils.nodes.footnote(auto=1)
        footnote['names'].append(name)
        footnote += self.paragraph(entry)
        document.note_autofootnote(footnote)
        document.note_explicit_target(footnote, footnote)
        return footnote

    def footnote_reference(self, entry: "FormattedEntry",
                           document: docutils.nodes.document
                           ) -> docutils.nodes.footnote_reference:
        """Return footnote_reference node to the given citation. The
        footnote_reference is expected to be inserted into *document*
        prior to any docutils transforms.

        .. versionadded:: 0.2.2
        """
        # see docutils.parsers.rst.states.Body.footnote_reference()
        refname = docutils.nodes.fully_normalize_name(entry.key)
        refnode = docutils.nodes.footnote_reference(
            '[#%s]_' % entry.key, refname=refname, auto=1)
        document.note_autofootnote_ref(refnode)
        document.note_footnote_ref(refnode)
        return refnode



from setuptools import setup
import codecs


def readfile(filename):
    with codecs.open(filename, encoding="utf-8") as stream:
        return stream.read().split("\n")


doclines = readfile("README.rst")[5:]  # skip badges
requires = readfile("requirements.txt")
version = readfile("VERSION")[0].strip()

setup(
    name='pybtex-docutils',
    version=version,
    url='https://github.com/mcmtroffaes/pybtex-docutils',
    download_url='http://pypi.python.org/pypi/pybtex-docutils',
    license='MIT',
    author='Matthias C. M. Troffaes',
    author_email='matthias.troffaes@gmail.com',
    description=doclines[0],
    long_description="\n".join(doclines[2:]),
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Text Editors :: Text Processing',
        'Topic :: Text Processing :: Markup :: XML'
    ],
    platforms='any',
    packages=['pybtex_docutils'],
    package_dir={'': 'src'},
    package_data={'pybtex_docutils': ['py.typed']},
    install_requires=requires,
    entry_points={
        'pybtex.backends': [
            'docutils = pybtex_docutils:Backend',
        ]
    },
    python_requires='>=3.6',
)



import docutils.nodes
import docutils.utils
import pytest
import pybtex.plugin
import pybtex.database
from pybtex.richtext import HRef, Tag, Text

from pybtex_docutils import Backend


def render_str(richtext):
    return "".join(str(node) for node in richtext.render(Backend()))


# may remove this test when new pybtex is out
def test_text():
    assert Backend().format_text('hi') == Backend().format_str('hi')


def test_tag():
    tag = Tag('em', 'hello')
    assert render_str(tag) == '<emphasis>hello</emphasis>'


def test_tag_text():
    tag = Tag('em', Text('hello', ' world'))
    assert render_str(tag) == '<emphasis>hello world</emphasis>'


def test_tag_strong():
    tag = Tag('strong', 'hello')
    assert render_str(tag) == '<strong>hello</strong>'


def test_tag_i():
    tag = Tag('i', 'hello')
    assert render_str(tag) == '<emphasis>hello</emphasis>'


def test_tag_b():
    tag = Tag('b', 'hello')
    assert render_str(tag) == '<strong>hello</strong>'


def test_tag_tt():
    tag = Tag('tt', 'hello')
    assert render_str(tag) == '<literal>hello</literal>'


def test_tag_sup():
    tag = Tag('sup', 'hello')
    assert render_str(tag) == '<superscript>hello</superscript>'


def test_tag_sub():
    tag = Tag('sub', 'hello')
    assert render_str(tag) == '<subscript>hello</subscript>'


def test_tag_unknown():
    tag = Tag('***unknown***', 'hello')
    assert render_str(tag) == 'hello'


def test_href():
    href = HRef('http://www.example.com', 'hyperlinked text')
    assert render_str(href) == (
        '<reference refuri="http://www.example.com">'
        'hyperlinked text'
        '</reference>')


def test_href_text():
    href = HRef('http://www.example.com', Text('hyperlinked', ' text'))
    assert render_str(href) == (
        '<reference refuri="http://www.example.com">'
        'hyperlinked text'
        '</reference>')


def test_render_sequence():
    text = Text('hello ', Tag('em', 'world'))
    assert render_str(text) == 'hello <emphasis>world</emphasis>'


@pytest.fixture
def entry():
    data = pybtex.database.BibliographyData({
        'hongquin1997': pybtex.database.Entry(
            'article',
            fields={
                'language': 'english',
                'title': 'Predicting the Diffusion Coefficient'
                         ' in Supercritical Fluids',
                'journal': 'Ind. Eng. Chem. Res.',
                'volume': '36',
                'year': '1997',
                'pages': '888-895',
            },
            persons={'author': [
                pybtex.database.Person('Liu, Hongquin'),
                pybtex.database.Person('Ruckenstein, Eli')]},
            )})
    style = pybtex.plugin.find_plugin('pybtex.style.formatting', 'plain')()
    entries = list(style.format_entries(data.entries.values()))
    return entries[0]


@pytest.fixture
def document():
    return docutils.utils.new_document('test.rst')


def test_citation(entry, document):
    node = Backend().citation(entry, document)
    assert str(node) == (
        '<citation ids="hongquin1997" names="hongquin1997">'
        '<label>hongquin1997</label>'
        '<paragraph>'
        'Hongquin Liu and Eli Ruckenstein. '
        'Predicting the diffusion coefficient in supercritical fluids. '
        '<emphasis>Ind. Eng. Chem. Res.</emphasis>, '
        '36:888–895, 1997.'
        '</paragraph>'
        '</citation>')


def test_citation_reference(entry, document):
    node = Backend().citation_reference(entry, document)
    assert str(node) == (
        '<citation_reference ids="id1" refname="hongquin1997">'
        'hongquin1997'
        '</citation_reference>')


def test_citation_use_label(entry, document):
    node = Backend().citation(
        entry, document, use_key_as_label=False)
    assert str(node) == (
        '<citation ids="hongquin1997" names="hongquin1997">'
        '<label>1</label>'
        '<paragraph>'
        'Hongquin Liu and Eli Ruckenstein. '
        'Predicting the diffusion coefficient in supercritical fluids. '
        '<emphasis>Ind. Eng. Chem. Res.</emphasis>, '
        '36:888–895, 1997.'
        '</paragraph>'
        '</citation>')


def test_citation_reference_use_label(entry, document):
    node = Backend().citation_reference(
        entry, document, use_key_as_label=False)
    assert str(node) == (
        '<citation_reference ids="id1" refname="hongquin1997">'
        '1'
        '</citation_reference>')


def test_footnote(entry, document):
    node = Backend().footnote(entry, document)
    assert str(node) == (
        '<footnote auto="1" ids="hongquin1997" names="hongquin1997">'
        '<paragraph>'
        'Hongquin Liu and Eli Ruckenstein. '
        'Predicting the diffusion coefficient in supercritical fluids. '
        '<emphasis>Ind. Eng. Chem. Res.</emphasis>, '
        '36:888–895, 1997.'
        '</paragraph>'
        '</footnote>')


def test_footnote_reference(entry, document):
    node = Backend().footnote_reference(entry, document)
    assert str(node) == (
        '<footnote_reference auto="1" ids="[\'id1\']" '
        'refname="hongquin1997"/>')


def test_write_entry():
    with pytest.raises(NotImplementedError):
        Backend().write_entry(None, None, None)
